/*
 * Copyright (c) 2000 Greg Haerr <greg@censoft.com>
 * Original contributions by Shane Nay
 *
 * Win32 API upper level font selection routines
 */
#include "windows.h"
#include "wintern.h"
#include <stdlib.h>

HFONT WINAPI
CreateFont(int nHeight, int nWidth, int nEscapement, int nOrientation,
	int fnWeight, DWORD fdwItalic, DWORD fdwUnderline, DWORD fdwStrikeOut,
	DWORD fdwCharSet,DWORD fdwOutputPrecision,DWORD fdwClipPrecision,
	DWORD fdwQuality, DWORD fdwPitchAndFamily, LPCSTR lpszFace)
{
	LOGFONT	lf;

	lf.lfHeight = nHeight;
	lf.lfWidth = nWidth;
	lf.lfEscapement = nEscapement;
	lf.lfOrientation = nOrientation;
	lf.lfWeight = fnWeight;
	lf.lfItalic = fdwItalic;
	lf.lfUnderline = fdwUnderline;
	lf.lfStrikeOut = fdwStrikeOut;
	lf.lfCharSet = fdwCharSet;
	lf.lfOutPrecision = fdwOutputPrecision;
	lf.lfClipPrecision = fdwClipPrecision;
	lf.lfQuality = fdwQuality;
	lf.lfPitchAndFamily = fdwPitchAndFamily;
	strcpy(lf.lfFaceName, lpszFace);

	return CreateFontIndirect(&lf);
}

HFONT WINAPI
CreateFontIndirect(CONST LOGFONT *lplf)
{
	FONTOBJ * 	hfont;

	/* create a gdi font object*/
	hfont = ItemNew(FONTOBJ);
	if(!hfont)
		return NULL;
	hfont->hdr.type = OBJ_FONT;
	hfont->hdr.stockobj = FALSE;

	/* select a font based on facename and height*/
	hfont->pfont = GdCreateFont(&scrdev, lplf->lfFaceName, lplf->lfHeight,
			(const PMWLOGFONT)lplf);

	return (HFONT)hfont;
}

BOOL WINAPI
GetTextMetrics(HDC hdc, LPTEXTMETRIC lptm)
{
	FONTINFO 	fi;

	if(!hdc)
		return FALSE;

	GdGetFontInfo(hdc->font->pfont, &fi);

	/* FIXME many items are guessed for the time being*/
	lptm->tmHeight = fi.height;
	lptm->tmAscent = fi.height - fi.baseline;
	lptm->tmDescent= fi.baseline;
	lptm->tmInternalLeading = 0;
	lptm->tmExternalLeading = 0;
	lptm->tmAveCharWidth = fi.widths['x'];
	lptm->tmMaxCharWidth = fi.maxwidth;
	lptm->tmWeight = FW_NORMAL;
	lptm->tmOverhang = 0;
	lptm->tmDigitizedAspectX = fi.maxwidth;
	lptm->tmDigitizedAspectY = fi.height;
	lptm->tmFirstChar = 32;
	lptm->tmLastChar = 255;
	lptm->tmDefaultChar = '?';
	lptm->tmBreakChar = 0;
	lptm->tmItalic = 0;
	lptm->tmUnderlined = 0;
	lptm->tmStruckOut = 0;
	/* note that win32 has the TMPF_FIXED_PITCH flags REVERSED...*/
	lptm->tmPitchAndFamily = fi.fixed?
			FF_DONTCARE: (FF_DONTCARE | TMPF_FIXED_PITCH);
	lptm->tmCharSet = OEM_CHARSET;
	return TRUE;
}

BOOL WINAPI
GetCharWidth(HDC hdc, UINT iFirstChar, UINT iLastChar, LPINT lpBuffer)
{
	int 		i;
	int		j = 0;
	FONTINFO	fi;

	if(!hdc || iLastChar < iFirstChar)
		return FALSE;

	GdGetFontInfo(hdc->font->pfont, &fi);
	for(i=iFirstChar; i <= iLastChar; ++i)
		if(i < fi.firstchar || i > fi.lastchar || i > 255)
			lpBuffer[j++] = 0;
		else lpBuffer[j++] = fi.widths[i];

	return TRUE;
}
